using System;
using System.Runtime.InteropServices;
using System.Text;
using System.IO;
using System.Collections.Generic;
using System.Globalization;
using System.Net;
using System.Net.Sockets;

namespace nsASASClient
{

  public class PrivateProfile
  {

    [DllImport("KERNEL32.DLL", EntryPoint = "GetPrivateProfileStringW",
      SetLastError = true,
      CharSet = CharSet.Unicode, ExactSpelling = true,
      CallingConvention = CallingConvention.StdCall)]
    private static extern int GetPrivateProfileString(
    string lpAppName,
    string lpKeyName,
    string lpDefault,
    string lpReturnString,
    int nSize,
    string lpFilename);

    [DllImport("KERNEL32.DLL", EntryPoint = "GetPrivateProfileIntW",
      SetLastError = true,
      CharSet = CharSet.Unicode, ExactSpelling = true,
      CallingConvention = CallingConvention.StdCall)]
    private static extern int GetPrivateProfileInt(
    string lpAppName,
    string lpKeyName,
    int iDefault,
    string lpFilename);

    [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
      SetLastError = true,
      CharSet = CharSet.Unicode, ExactSpelling = true,
      CallingConvention = CallingConvention.StdCall)]
    private static extern int WritePrivateProfileString(
    string lpAppName,
    string lpKeyName,
    string lpString,
    string lpFilename);

    public static string ApplicationPath()
    {
#if DEBUG
      return "..\\..";
#else
			string AssemblyName = System.Reflection.Assembly.GetExecutingAssembly().Location;
      System.IO.FileInfo file = new System.IO.FileInfo(AssemblyName);
      return file.Directory.FullName; ;
#endif

    }

    public static string DataPath()
    {
      return Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + "\\" + ASASClient.CompanyName + "\\ASASClient\\" + PrivateProfile.GetIniFileString("General", "ServiceName", "unknown");
    }

    public static string SettingsPath()
    {
			return ApplicationPath();
    }

    public static string SysLogFilesPath()
    {
      return DataPath() + "\\SysLogFiles";
    }

    public static string DebugFilesPath()
    {
      return DataPath() + "\\DebugFiles";
    }

    public static string IniFileFullname()
    {
      return SettingsPath() + "\\" + "ASASClient.ini";
    }

    public static string GetIniFileString(string category, string key, string defaultValue)
    {
      string returnString = new string(' ', 1024);
      GetPrivateProfileString(category, key, defaultValue, returnString, 1024, IniFileFullname());
      return returnString.Split('\0')[0];
    }

    public static int GetIniFileInt(string category, string key, int defaultValue)
    {
      return GetPrivateProfileInt(category, key, defaultValue, IniFileFullname());
    }

    public static void WriteIniFileString(string category, string key, string value)
    {
      WritePrivateProfileString(category, key, value, IniFileFullname());
    }

    public static void WriteIniFileInt(string category, string key, int value)
    {
      WritePrivateProfileString(category, key, value.ToString(), IniFileFullname());
    }

    public static List<string> GetCategories(string iniFile)
    {
      string returnString = new string(' ', 65536);
      GetPrivateProfileString(null, null, null, returnString, 65536, iniFile);
      List<string> result = new List<string>(returnString.Split('\0'));
      result.RemoveRange(result.Count - 2, 2);
      return result;
    }

    public static List<string> GetKeys(string iniFile, string category)
    {
      string returnString = new string(' ', 32768);
      GetPrivateProfileString(category, null, null, returnString, 32768, iniFile);
      List<string> result = new List<string>(returnString.Split('\0'));
      result.RemoveRange(result.Count - 2, 2);
      return result;
    }

    public static string Item(string sString, int iItem, char chDelimiter)
    {

      string[] sItems = sString.Split(chDelimiter);

      if (sItems.GetLength(0) > iItem)
      {
        return sItems[iItem];
      }
      else
      {
        return "";
      }
    }

  }

  public class SysLogAndDebug
  {

		public int DaysToKeepLogFiles = PrivateProfile.GetIniFileInt("General", "DaysToKeepLogFiles", 365);

    public const int Direction_ASASServer2Client = 0;
    public const int Direction_Client2ASASServer = 1;

    public string sSysLogFilePath;
    public string sDebugFilePath;

		public int LastCleanupDay;
    public int DebugFileCloseTimer = 0;

    public SysLogAndDebug()
    {
			sSysLogFilePath = PrivateProfile.SysLogFilesPath();
      sDebugFilePath = PrivateProfile.DebugFilesPath();
    }

		// Delete old logfiles
		public void CyclicCleanup(int iElapsedMillisecs)
		{
			if (LastCleanupDay != DateTime.Now.Day)
			{
				DeleteLogFiles (sSysLogFilePath, "SysLog_????????.Log");
        DeleteLogFiles (sDebugFilePath, "Debug_????????.Log");
        LastCleanupDay = DateTime.Now.Day;
			}
      // Close file cyclically...
      if (ASASClient.swDebugFile != null)
      {
        DebugFileCloseTimer += iElapsedMillisecs;
        if (DebugFileCloseTimer >= 1000)
        {
          lock (this)
          {
            ASASClient.swDebugFile.Close();
            ASASClient.swDebugFile = null;
            DebugFileCloseTimer = 0;
          }
        }
      }
		}

		public void DeleteLogFiles(string LogFilePath, string LogFileName)
		{
      IFormatProvider culture = new CultureInfo("en-US");

      string[] LogFiles = Directory.GetFiles(LogFilePath, LogFileName);

			foreach (string LogFile in LogFiles)
			{
        if (LogFile.Length > 12)
        {
          string sDateTime = LogFile.Substring(LogFile.Length - 12, 8);
          try
          {
            DateTime LogFileDateTime = DateTime.ParseExact(sDateTime, "yyyyMMdd", culture);
            if (LogFileDateTime.AddDays(DaysToKeepLogFiles).CompareTo(DateTime.Now) < 0)
            {
              File.Delete(LogFile);
            }
          }
          catch { }
        }
			}
		}

    public void SysLog(string sFormat, params object[] pArg)
    {

      string sLogText = String.Format("{0:HH:mm:ss}", DateTime.Now) + " " + String.Format(sFormat, pArg);
			string sFileName = sSysLogFilePath + "\\SysLog_" + String.Format("{0:yyyyMMdd}", DateTime.Now) + ".Log";

      sLogText = sLogText.Replace('\x7f', '?');
      sLogText = sLogText.Replace('\n', '?');
      sLogText = sLogText.Replace('\r', '?');
      sLogText = sLogText.Replace('\t', '?');

      lock (this)
      {

        Console.WriteLine(String.Format("{0:yyyy-MM-dd}", DateTime.Now) + " " + sLogText);

        try
        {
          StreamWriter swSysLogFile = File.AppendText(sFileName);
          if (sFormat.Length == 0)
          {
            swSysLogFile.WriteLine("------------------------------------------------------------------------------------------------------------------------");
          }
          else
          {
            swSysLogFile.WriteLine(sLogText);
          }
          swSysLogFile.Close();
        }
        catch { }

      }
    }

    public void Debug(int iDirection, string sDestinationAddress, byte[] bBuffer, int iOffset, int iBufferLength, string sBuffer, bool bForceBinary)
    {

      string ExpandedBuffer = "";
      string sDebugText;
      string sSingleRow;

      if (PrivateProfile.GetIniFileInt("General", "ContinousDebug", 0) == 0)
      {
        return;
      }

      // Could be byte buffer to be debugged...
      if (iBufferLength > 0 && bBuffer != null)
      {
        for (int iCharacter = iOffset; iCharacter < (iOffset + iBufferLength); iCharacter++)
        {
          if (((bBuffer[iCharacter] < ' ' || bBuffer[iCharacter] > '\x7f') && bBuffer[iCharacter] != '\n') || bForceBinary)
          {
            ExpandedBuffer = ExpandedBuffer + "<" + bBuffer[iCharacter].ToString("D") + ">";
          }
          else
          {
            ExpandedBuffer += Encoding.Default.GetString(bBuffer, iCharacter, 1);
          }
        }
      }

      // ...or string buffer
      if (sBuffer != null)
      {
        ExpandedBuffer += sBuffer;
      }

      string HeaderText = sDestinationAddress + "\t";

      switch (iDirection)
      {
        case Direction_ASASServer2Client:
					HeaderText += "ASAS -> Client\t";
					break;
        case Direction_Client2ASASServer:
          HeaderText += "ASAS <- Client\t";
					break;
      }

      bool bFirstRow = true;

      while (ExpandedBuffer.Length > 0)
      {

        int iNextLF = ExpandedBuffer.IndexOf('\n');

        if (iNextLF >= 0)
        {
          sSingleRow = ExpandedBuffer.Substring(0, iNextLF) + "<10>";
          try
          {
            ExpandedBuffer = ExpandedBuffer.Substring(iNextLF + 1);
          }
          catch
          {
            ExpandedBuffer = "";
          }
        }
        else
        {
          sSingleRow = ExpandedBuffer;
          ExpandedBuffer = "";
        }

        if (bFirstRow)
        {
          sDebugText = String.Format("{0:yyyy-MM-dd}", DateTime.Now) + "\t" + String.Format("{0:HH:mm:ss}", DateTime.Now) + "\t";
        }
        else
        {
          sDebugText = "\t\t";
        }

        // Does not look good in Notepad if date/time is omitted in following rows...

        sDebugText += HeaderText + sSingleRow + "\r\n";

        lock (this)
        {
          try
          {
            if (ASASClient.swDebugFile == null)
            {
              string sFileName = sDebugFilePath + "\\Debug_" + String.Format("{0:yyyyMMdd}", DateTime.Now) + ".Log";
              ASASClient.swDebugFile = File.AppendText(sFileName);
            }
            ASASClient.swDebugFile.Write(sDebugText);
          }
          catch {}
        }
      }
    }
  }

	public class TCPHelper
	{

		public static void CloseAndDeleteStreamAndSocket(ref NetworkStream networkStream, ref TcpClient tcpClient)
		{
			if (networkStream != null)
			{
				try
				{
					networkStream.Close();
					networkStream = null;
				}
				catch
				{
				}
			}
			if (tcpClient != null)
			{
				try
				{
					tcpClient.Close();
					tcpClient = null;
				}
				catch
				{
				}
			}
		}

		public static string UTF8toIsoLatin1(string InString)
		{

			string OutString = InString;

			Encoding EncodingIso = Encoding.GetEncoding("ISO-8859-1");
			Encoding EncodingUTF8 = Encoding.UTF8;

			byte[] UTF8Bytes = EncodingIso.GetBytes(OutString);
			byte[] IsoBytes = Encoding.Convert(EncodingUTF8, EncodingIso, UTF8Bytes);

			OutString = EncodingIso.GetString(IsoBytes);

			return OutString;

		}

		public static string DOSAscii2ISOLatin1(string InString)
		{

			string OutString = InString;

			OutString = OutString.Replace("\x8e", "");
			OutString = OutString.Replace("\x8f", "");
			OutString = OutString.Replace("\x99", "");

			OutString = OutString.Replace("\x84", "");
			OutString = OutString.Replace("\x86", "");
			OutString = OutString.Replace("\x94", "");

			OutString = OutString.Replace("\xf8", "");

			return OutString;

		}

    public static bool ResolveServerAddress(string sServerAddressAndPort, out IPAddress ipServerAddress, out int iServerPort)
    {

      ipServerAddress = null;
      iServerPort = 0;

      try
      {

        iServerPort = Int32.Parse(PrivateProfile.Item(sServerAddressAndPort, 1, ':'));
        if (iServerPort > 0 && iServerPort < 65536)
        {

          IPAddress[] ips;
          ips = Dns.GetHostAddresses(PrivateProfile.Item(sServerAddressAndPort, 0, ':'));

          // Lookup some IPv4 address (fails in Windows 7 otherwise)
          foreach (IPAddress ip in ips)
          {
            if (ip.AddressFamily == AddressFamily.InterNetwork)
            {
              ipServerAddress = ip;
              break;
            }
          }
        }
      }
      catch
      {
        ipServerAddress = null;
        iServerPort = 0;
      }

      return (ipServerAddress == null) ? false : true;

    }
	}

}
